issue_errors = r"""
        (
            (
                For\s+our\s+resolution
            )|
            (
                For\s+the\s+resolution\s+of\s+this\s+Court\s+then
            )|
            (
                For\s+this\s+Court[’']s\s+resolution
            )|
            (
                [Aa]ppellants?\s+assign
                (
                    s|ed
                )?
            )|
            (
                asked\s+to\s+resolve
            )|
            (
                anchored\s+on
            )|
            (
                based\s+on
            )|
            (
                submit # submits, submitted, submitting
                (
                    s|
                    ted|
                    ting
                )
            )|
            (
                rais # raise, raises, raised, raising
                (
                    e|
                    es|
                    ed|
                    ing
                )
            )|
            (
                present # presents, presented, presenting
                (
                    es|
                    ed|
                    ing
                )
            )
        )
        \s+
        (
            (
                are|
                is
            )\s+
        )?
        the\s+
        (
            (
                following|
                sole|
                lone
            )\s+
        )?
        (as\s+)?
        (
            (issues?)|
            (assigned\s+errors?)|
            (errors?)|
            (assignments?\s+of\s+errors?)|
            (arguments?)
        )
        \:?
    """



main_issue = r"""
        (
            (^\s* # possible space before "The|Thus"
                (
                    The|
                    Thus,\s+the|
                )
            )|
            (
                raising\s+
                the
            )|
            (
                may\s+
                be\s+
                reduced\s+
                to
            )|
            (
                for\s+
                Our\s+
                consideration
                ,
            )|
            Apparently,|
            Appropriately,|
            Essentially,|
            Basically,|
            Simply,
        )\s+
        (the\s+)?
        (
            principal|
            threshold|
            core|
            pivotal|
            sole|
            main
        )\s+
        (
            issue|
            question
        )
    """



from bs4 import NavigableString, Tag

from acquisition_ruling_phrase.utils import PATTERN


def center_next_tag(t: Tag):
    return (
        t.name == "center"
        and isinstance(t.next_element, Tag)
        and PATTERN.search(t.text)
        and (t.next_element.name == "strong" or t.next_element.name == "em")
        and len(t.text.strip()) < 25  # text is short
    )


def center_next_string(t: Tag):
    return (
        t.name == "center"
        and isinstance(t.next_element, NavigableString)
        and PATTERN.search(t.text)
        and len(t.text.strip()) < 25
    )


def p_strong(t: Tag):
    # re: center in attrs... this deals with odd <p align="“center”">
    return (
        t.name == "p"
        and t.attrs.get("align")
        and "center" in t.attrs.get("align")
        and isinstance(t.next_element, Tag)
        and t.next_element.name == "strong"
        and PATTERN.search(t.text)
        and len(t.text.strip()) < 25
    )


def p_space_strong(t: Tag):
    # re: center in attrs... this deals with odd <p align="“center”">
    return (
        t.name == "p"
        and t.attrs.get("align")
        and "center" in t.attrs.get("align")
        and isinstance(t.next_element, NavigableString)
        and len(str(t.next_element).strip()) == 0
        and isinstance(t.next_element.next_element, Tag)
        and t.next_element.next_element.name == "strong"
        and PATTERN.search(t.text)
        and len(t.text.strip()) < 25
    )



court_acts = r"""
        ^\s* # possible space before "The"
        (
            (This\s+Court)|
            (The\s+Court)|
            We
        )\s+
        (
            den(y|ies)|
            finds?|
            grants?
        )\s+
        (
            (no\s+)?
            merit\s+
            in\s+
        )?
        (th(e|is)\s+)?
        (
            instant\s+|
            present\s+
        )?
        (petitioners'\s+)?
        (
            [Pp]etition|
            [Aa]ppeal|
            motion|
            contention
        )
        \s*
        (to\s+be)?
        (without\s+merit)?
        (partially\s+with\s+merit)?
        (for\s+lack\s+of\s+merit)?
    """



from typing import Callable, Optional

from bs4 import BeautifulSoup

from .cues import ruling_cue
from .dispositive.phrases import Phrase
from .tags import (
    center_next_string,
    center_next_tag,
    p_space_strong,
    p_strong,
    proper_words,
)


def recipe(func: Callable, raw: str) -> Optional[str]:
    """Sanitize and use specific function"""
    soup = BeautifulSoup(raw, "html5lib")
    results = soup(func)
    if not results:
        return None
    return str(results[-1])  # the last one has the least tags


def fetch(text: str) -> Optional[str]:
    """Attempt either (1) tagged phrase via recipe; or (2) textual cues"""
    return (
        recipe(center_next_tag, text)
        or recipe(center_next_string, text)
        or recipe(p_strong, text)
        or recipe(p_space_strong, text)
        or recipe(proper_words, text)
        or ruling_cue(text)
    )


def wherefore_clause(context: str) -> Optional[str]:
    return Phrase("WHEREFORE", context).get_clause or None


def ordered_clause(context: str) -> Optional[str]:
    return Phrase("ORDERED", context).get_clause or None



import re

from bs4 import NavigableString, Tag

RULING_TEXT = r"""
    ^
    \s*
    (
        (Our\s+)?Ruling|
        (OUR\s+)?RULING|
        (The\s+)?Ruling\s+of\s+the\s+Court|
        RULING\s+OF\s+THE\s+COURT|
        The\s+Court[’']s\s+Ruling
    )
    \.?
    \s*
    $
"""

PATTERN = re.compile(RULING_TEXT, re.X)


def non_lowered_tagged_text_short(t: Tag) -> bool:
    """
    Used in both Ruling and Phrases to determine whether or not a tag looks like a section header
    1. <b>'s next elemnt is "So Ordered", the next element is a string rather than a tag
    2. "So Ordered" is more than 8 characters, less than 50, the navigable string is short
    3. "so ordered" is excluded; exclude cases like: (1) '<u>ruling</u> '; (2) <i>decision, order, or ruling</i>
    """
    text = t.get_text()
    count = len(text)
    return (
        isinstance(t.next_element, NavigableString)
        and (count > 5 and count < 50)
        and not text.islower()
    )



from dataclasses import dataclass
from typing import Match, Optional

from bs4 import BeautifulSoup, NavigableString, Tag

from acquisition_ruling_phrase.utils import non_lowered_tagged_text_short

from .ordered import ordered_pattern
from .wherefore import wherefore_pattern


@dataclass
class Phrase:
    """
    Class for determining WHEREFORE and ORDERED clauses

    1. String matching pattern is contained inside a tag
    2. String matching pattern is found after a tag
    3. String matching pattern is behind a tag

    The return is a stringified HTML e.g. "<b>WHEREFORE</b>"
    """

    mode: str  # Mode determines what pattern to use
    text: str  # Context to search the pattern

    @property
    def pattern(self):
        if self.mode == "WHEREFORE":
            return wherefore_pattern
        elif self.mode == "ORDERED":
            return ordered_pattern

    @property
    def get_clause(self) -> Optional[str]:
        html = BeautifulSoup(self.text, "html5lib")
        return (
            self.within_tag(html)
            or self.succeed_tag(html)
            or self.precede_tag(html)
        )

    def last(self, html: BeautifulSoup, t: Tag) -> Optional[Tag]:
        """Does target exist in html? If yes, return last tag"""
        return match[-1] if (match := html(t)) else None

    def within_tag(self, html: BeautifulSoup) -> Optional[str]:
        """e.g. <b> SO ORDERED. </b>"""
        return str(res) if (res := self.last(html, self.inside_tag)) else None

    def succeed_tag(self, html: BeautifulSoup) -> Optional[str]:
        """e.g. <br>"So Ordered." """
        match = self.last(html, self.tag_then_text)
        return str(match.next_element) if match else None

    def precede_tag(self, html: BeautifulSoup) -> Optional[str]:
        """e.g. "So Ordered <br>"""
        match = self.last(html, self.pattern_precedes_tag)
        return str(match.previous_element) if match else None

    def inside_tag(self, t: Tag):
        return (
            t.string
            and self.pattern.search(t.get_text())
            and non_lowered_tagged_text_short(t)
        )

    def tag_then_text(self, t: Tag):
        if isinstance(t.next_element, NavigableString):
            return self.pattern.search(t.next_element)
        return None

    def pattern_precedes_tag(self, t: Tag) -> Optional[Match]:
        # the previous element of a tag is a string which contains an ordered clause
        if isinstance(t.previous_element, NavigableString):
            return self.pattern.search(t.previous_element)
        return None



single_issue = r"""
    ^
    \s*
    (
        (Verily,\s+the)|
        (The)
    )
    \s+
    (
        (
            main|
            basic
        )
        \s+
    )?
    issue
    \s+
    (
        (in\s+this\s+case)|
        (is\s+whether\s+or\s+not)
    )
"""



after_review = r"""
    (
        (After\s+a)|
        Upon
    )
    \s+
    (
        careful|
        thorough
    )
    \s+
    review
    \s+
    of
    \s+
    the
    \s+
    (
        record|
        petition
    )s?
"""



import re
from typing import Optional

from bs4 import BeautifulSoup

from .patterns import (
    after_review,
    court_acts,
    court_admin,
    granted_denied,
    hence_this,
    issue_errors,
    issue_resolution,
    main_issue,
    merit,
    resolve_issue,
    single_issue,
)


def ruling_cue(text: str) -> Optional[str]:
    """Get first matching cue"""
    html = BeautifulSoup(text, "html5lib")
    cues = iter(
        [
            after_review,
            court_acts,
            court_admin,
            granted_denied,
            hence_this,
            issue_errors,
            issue_resolution,
            main_issue,
            merit,
            resolve_issue,
            single_issue,
        ]
    )

    for cue in cues:
        pattern = re.compile(cue, re.X)
        if matches := html(string=pattern):
            initial = matches[0]
            return str(initial)

    return None



granted_denied = r"""
        ^\s* # possible space before "The"
        (
            The|
            Appellants\’|
            [A-Z][a-z]+\’s # Amistoso's
        )
        \s+
        (
            recourse|
            appeals?|
            petitions?|
            contentions?
        )
        \s+
        (
            is|
            are|
            must
        )
        \s+
        (
            fail|
            baseless|
            denied|
            (
                without|
                (
                    (not\s+)?
                    impressed
                    \s+
                    with
                )|
                (
                    bereft
                    \s+
                    of
                )
                \s+
                merit
            )|
            (
                (
                    partly\s+
                )?
                granted
            )|
            (
                (
                    un|
                    partly\s+
                )?
                meritorious
            )
        )
    """



from typing import Optional

from bs4 import BeautifulSoup, Tag

from acquisition_ruling_phrase.utils import (
    PATTERN,
    non_lowered_tagged_text_short,
)


def tagged_acquisition_ruling_phrases(t: Tag):
    """Short, no lower case, tag next to string"""
    return (
        non_lowered_tagged_text_short(t)
        and [inside for inside in (c for c in t.children) if inside.string]
        and PATTERN.search(t.get_text())
    )


def words_inside(tag: Tag) -> Optional[Tag]:
    """
    Some tags with rulings have words that disqualify it from becoming the
    start index. Remove these tags from the list based on looking for words
    that do not fall under the common style # see https://stackoverflow.com/a/25346119
    """
    words_in_tag = tag.get_text().lower().split()
    proper = [
        "ruling",
        "our",
        "this",
        "court",
        "court’s",
        "court's",
        "the",
        "of",
    ]  # note "courts"
    flagged_as_improper = [w for w in words_in_tag if w not in proper]
    return tag if not flagged_as_improper else None


def proper_words(html: BeautifulSoup) -> Optional[str]:
    """
    Get phrase to use as start of slice
    1. first check: does tag qualify?
    2. second check: does it contain only the proper words?
    """
    qualified = html(tagged_acquisition_ruling_phrases)
    if not qualified:
        return None

    proper = [tag for tag in qualified if words_inside(tag)]
    if not proper:
        return None

    return str(proper[0])



from .after_review import after_review
from .court_acts import court_acts
from .court_admin import court_admin
from .granted_denied import granted_denied
from .hence_this import hence_this
from .issue_errors import issue_errors
from .issue_resolution import issue_resolution
from .main_issue import main_issue
from .merit import merit
from .resolve_issue import resolve_issue
from .single_issue import single_issue



court_admin = r"""
        ^\s* # possible space before "The"?
        (
            (The\s+Court)|
            (After\s+a\s+careful\s+review\s+of\s+the\s+records\s+of\s+this\s+case,\s+the\s+Court)|
            We
        )\s+
        (
            (find\s+no\s+reason\s+to\s+disturb)|
            (are\s+in\s+full\s+accord)|
            (note\s+and\s+adopt)|
            (see\s+no\s+reason\s+to\s+depart\s+from)|
            agrees?|
            (agrees?\s+and\s+adopts?)|
            (agrees?\s+with,\s+and\s+adopts?)|
            adopts?|
            (adopts?\s+and\s+agrees?)|
            concurs?|
            approve
        )\s+
        (in\s+toto\s+)?
        (with\s+)?
        (the\s+)?
        (
            (
                findings?\s+
                (and\s+recommendations?\s+)?
            )|
            (
                recommendations?\s+
            )|
            (
                foregoing\s+observations\s+
            )|
            (
                findings?\s+
                (and\s+conclusions?\s+)?
            )
        )?
        (
            of
            ,?
            \s+
        )?
        (the\s+)?
        (
            IBP|
            OCA|
            (
                (
                    Office\s+of\s+the\s+
                )?
                Court\s+Administrator
            )
        )
        (
            's\s+
            (
                findings?\s+
                (and\s+recommendations?\s+)?
            )|
            (
                recommendations?\s+
            )
        )?
    """



import re

ordered_regex = r"""
    (
        (SO)?
        \s*
        ORE?DERED? # see http://elibrary.judiciary.gov.ph/thebookshelf/showdocsfriendly/1/64960 ORDERED vs. SO ORDERED
    ) # see misspelt SO OREDERED. in http://elibrary.judiciary.gov.ph/thebookshelf/showdocsfriendly/1/34304
    \s*
    \.? # http://elibrary.judiciary.gov.ph/thebookshelf/showdocsfriendly/1/61481 OPTIONAL period
    """

ordered_pattern = re.compile(ordered_regex, re.X)



resolve_issue = r"""
        ^\s* # possible space before "The"
        (
            The|
            In|
            Essentially,\s+the|
            Summed\s+up,\s+the|
        )\s+
        issues?\s+
        (
            in\s+th(ese|is)|
            for(\s+our)?|
            to\s+be
        )\s+
        (
            petitions?|
            resolution|
            resolved
        )\s+
        (in\s+the\s+present\s+case\s+)?
        (is|are)?
    """



hence_this = r"""
        ^\s* # possible space before "Hence"
        Hence
        ,? # comma optional
        \s+
        (
            (
                th(e|is) # the / this
            )|
            petitioners'
        )
        \s+
        (
            (
                instant|
                present
            )\s+
        )?
        (
            [Pp]etition|
            [Aa]ppeal|
            [Rr]ecourse
        )
        (
            \s+ # might end in petition, e.g. hence this petition.
            (
                (,\s*with\s+the\s+following\s+issues?)|
                (for\s+review\s+on\s+certiorari)
            )?
        )?
    """



import re

wherefore_regex = r"""
\s*
(
    (Where?fore)|
    (WHERE?FORE)|
    (ACCORDINGLY)| # ACCORDINGLY examples, http://elibrary.judiciary.gov.ph/thebookshelf/showdocsfriendly/1/63290; http://elibrary.judiciary.gov.ph/thebookshelf/showdocsfriendly/1/64934
    (IN\s+VIEW\s+ # IN VIEW WHEREOF http://elibrary.judiciary.gov.ph/thebookshelf/showdocsfriendly/1/33483
        (
            (W?HEREOF)| # IN VIEW HEREOF http://elibrary.judiciary.gov.ph/thebookshelf/showdocsfriendly/1/33471
            (THEREOF)| # IN VIEW THEREOF http://elibrary.judiciary.gov.ph/thebookshelf/showdocsfriendly/1/34340
            (OF\s+(ALL\s+)?THE\s+FOREGOING)| # IN VIEW OF ALL THE FOREGOING http://elibrary.judiciary.gov.ph/thebookshelf/showdocsfriendly/1/64489
            (OF\s+THESE\s+CONSIDERATIONS) # http://elibrary.judiciary.gov.ph/thebookshelf/showdocsfriendly/1/33679
        )
    )|
    (ON\s+ALL\s+THE\s+FOREGOING\s+CONSIDERATIONS)| # http://elibrary.judiciary.gov.ph/thebookshelf/showdocsfriendly/1/33964
    (CONSIDERING\s+THE\s+FOREGOING)| # http://elibrary.judiciary.gov.ph/thebookshelf/showdocsfriendly/1/33587, http://elibrary.judiciary.gov.ph/thebookshelf/showdocsfriendly/1/34150, http://elibrary.judiciary.gov.ph/thebookshelf/showdocsfriendly/1/34759
    (PREMISES CONSIDERED)| # http://elibrary.judiciary.gov.ph/thebookshelf/showdocsfriendly/1/33729
    (FOR\s+THE\s+FOREGOING\s+REASONS)| # http://elibrary.judiciary.gov.ph/thebookshelf/showdocsfriendly/1/33600
    (UPON\s+THESE\s+PREMISES) # http://elibrary.judiciary.gov.ph/thebookshelf/showdocsfriendly/1/34578
)
\,?
\s*
"""

wherefore_pattern = re.compile(wherefore_regex, re.X)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['acquisition_ruling_phrase',
 'acquisition_ruling_phrase.dispositive',
 'acquisition_ruling_phrase.patterns',
 'acquisition_ruling_phrase.tags']

package_data = \
{'': ['*']}

install_requires = \
['acquisition-sanitizer>=0.4.1,<0.5.0',
 'beautifulsoup4>=4.10,<5.0',
 'html5lib>=1.1,<2.0']

setup_kwargs = {
    'name': 'acquisition-ruling-phrase',
    'version': '0.3.1',
    'description': 'Pattern matching common phrases in Supreme Court decisions indicating the start of content with doctrinal value.',
    'long_description': 'None',
    'author': 'Marcelino G. Veloso III',
    'author_email': 'mars@veloso.one',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



merit = r"""
        ^\s* # possible space before "The"
        Th(e|is)\s+
        (
            appeal|
            petition|
            argument
        )\s+
        (
            lacks|
            has\s+no|
            (is\s+
                (
                    without|
                    devoid\s+of|
                    bereft\s+of
                )
            )
        )\s+
        merit
    """
